package org.greenstone.gsdl3_extension.mat;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;


import java.io.*;

import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.greenstone.gsdl3.core.MessageRouter;
import org.greenstone.gsdl3.util.GlobalProperties;
import org.greenstone.gsdl3.util.GSFile;
import org.greenstone.gsdl3.util.GSXML;
import org.greenstone.gsdl3.util.XMLConverter;
import org.greenstone.gsdl3.util.GDBMWrapper;


public class DescribeMessenger {
    
    protected String site_name = "localsite";
    
    
    /** message router object to pass requests messages to and which 
     * will process them.*/
    protected MessageRouter mr = null;
    private Node CollectionList = null;
    /** container Document to create XML Nodes */
    protected Document doc=null;
    /** a converter class to parse XML and create Docs */
    protected XMLConverter converter=null;
    
    protected final static String mrSubsetOptions = //messageRouter 
	"collectionList serviceClusterList serviceList siteList"; 
    protected final static String csSubsetOptions = //collections and serviceClusters 
	"metadataList serviceList displayItemList";
    protected final static String serviceSubsetOptions = //services
	"paramList displayItemList";
    final int MESSAGEROUTER = 1, COLLECTION = 2, 
	SERVICECLUSTER = 3, SERVICE = 4;
    
    private String collection_Name = null;
    private String OAI_URL = null;
    protected GDBMWrapper gdbm_src = null;
    protected String site_name1 = "";
    protected String coll_name1 = "";
    protected Element coll_config_xml = null;
    protected HashMap element_map = new HashMap();
    protected MetadataStats ms;
    String site_home = null;
    
    public DescribeMessenger() {

	String gsdl3_home = GlobalProperties.getGSDL3Home();

	if (gsdl3_home == null || gsdl3_home.equals("")) {
	    System.err.println("Couldn't access GSDL3Home from GlobalProperties.getGSDL3HOME, can't initialize the SOAP Server.");
	    return;
	}

	
	site_home = GSFile.siteHome(gsdl3_home, this.site_name);

	File site_file = new File(site_home);
	if (!site_file.isDirectory()) {
	    System.err.println("The site directory "+site_file.getPath()+" doesn't exist. Can't initialize the SOAP Server.");
	    return;
	} 
	this.converter = new XMLConverter();
	this.doc = this.converter.newDOM();

	mr = new MessageRouter();

	mr.setSiteName(this.site_name);
	mr.setLibraryName(this.site_name);
	mr.configure();
	this.gdbm_src = new GDBMWrapper();


	
    }
    
    public DescribeMessenger(String collectionName, String URL) {
    	
    	collection_Name = collectionName;
    	OAI_URL = URL;
    	
    	String gsdl3_home = GlobalProperties.getGSDL3Home();
    	if (gsdl3_home == null || gsdl3_home.equals("")) {
    	    System.err.println("Couldn't access GSDL3Home from GlobalProperties.getGSDL3HOME, can't initialize the SOAP Server.");
    	    return;
    	}
    	
    	site_home = GSFile.siteHome(gsdl3_home, this.site_name);
    	
    	File site_file = new File(site_home);
    	if (!site_file.isDirectory()) {
    	    System.err.println("The site directory "+site_file.getPath()+" doesn't exist. Can't initialize the SOAP Server.");
    	    return;
    	} 
    	this.converter = new XMLConverter();
    	this.doc = this.converter.newDOM();
    	
    	mr = new MessageRouter();
    	mr.setSiteName(this.site_name);
    	//mr.configure();
    	this.gdbm_src = new GDBMWrapper();
    	//describe();
    }
    /*
    public void setURL(String arg){
    	OAI_URL = arg;
    }
    */
    
    public void setCollectionName(String arg){
    	collection_Name = arg;
    }
    
    public String describe() 
    {
	return describe("", "", "", mrSubsetOptions);
    }
    
    /*
    public String getURL(){
    	return OAI_URL;
    }
    */
    
    public String getCollectionName(){
    	return collection_Name;
    }
    
    /*
    //Describe Messages sent to Services (or Collection/Services) 
    public String describeCollService(String toColl, String toService, 
				      String lang, String subsetOption) {

	return describe(toColl + "/" + toService, 
			lang, subsetOption, serviceSubsetOptions);
    }
    */
    //If public, this method would give full access: a describe message that 
    //lets the user specify all the details of who the receiver is, and what 
    //details are requested.
    protected String describe(String to, String lang, 
			      String subsetOption, String validSubsetOptions)
    {
    	
	//Create message element: <message></message>
	try{
	    //DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    //Get the DocumentBuilder
	    //DocumentBuilder parser = factory.newDocumentBuilder();
	    //Create blank DOM Document
	    //Document doct = parser.newDocument();
	
	    Node message = doc.createElement(GSXML.MESSAGE_ELEM);
	    
	    //<message><request lang="en" to="" type="describe" uid="" /></message>
	    
	    Node request = GSXML.createBasicRequest(
						       this.doc, GSXML.REQUEST_TYPE_DESCRIBE, to, lang, "");
	    
	    //Check if only a subset of this Module Interface's data is asked
	    //to be described
	    
	    if(subsetOption != "") {
		//Now deal with the value for subset param:
		//only deal with valid params for subset of to-ModuleInterface
		if(validSubsetOptions.indexOf(subsetOption) == -1)
		    return "Invalid List to be described. Choose one of:\n"
			+ validSubsetOptions;
		
		//else, append <paramList />
		Node param_list = this.doc.createElement(
							    GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
		param_list.appendChild(GSXML.createParameter(
							     this.doc, GSXML.SUBSET_PARAM, subsetOption));
		request.appendChild(param_list);
		
	    }
	    message.appendChild(request);
	    //System.out.println(this.converter.getPrettyString(message));
	    /*
	    Element request = GSXML.createBasicRequest(
			       this.doc, GSXML.REQUEST_TYPE_PROCESS,"nwgfril"+"/DocumentMetadataRetrieve","en", "");

Element param_list = this.doc.createElement(
				GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);

param_list.appendChild(GSXML.createParameter(
				 this.doc, "metadata", "all"));

Element documentNode_list = this.doc.createElement(
	       GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);

Element documentNode = this.doc.createElement(GSXML.DOC_NODE_ELEM);
documentNode.setAttribute("nodeID","HASH01981d6a4a1502253e1a2283");
documentNode_list.appendChild(documentNode);

request.appendChild(param_list);
request.appendChild(documentNode_list);
message.appendChild(request);
System.out.println(this.converter.getPrettyString(message));
*/
	   // message.appendChild(request);
	    //System.out.println("\nRequest message");
	    //System.out.println(this.converter.getPrettyString(message));
	    //System.out.println(this.converter.getPrettyString(message));
	    //////////////////////////////////////////////////////////////////////////////////////////////
	    ////////////////////////////////////////////////////////////////////////////////////////////////
	    CollectionList = mr.process(message);
	    //System.out.println(this.converter.getPrettyString(CollectionList));
	    //Element response = mr.process(message);
	    //System.out.println(this.converter.getPrettyString(response));
	    //System.out.println("list"+CollectionList);
	    return ">>"+GSXML.MESSAGE_ELEM+"<<";
	    //return request.;
	}catch(Exception ex){return "failure";}
	//return this.converter.getPrettyString(message.toString());
	//return this.converter.getPrettyString(CollectionList);

    }
    
    public Node getCollectionList(){
    	return CollectionList;
    }


    public void describeMatadata(String collectionName,String OAIURL){

	ms = new MetadataStats(this.site_home,collectionName,OAIURL,"test");
	
	if(ms.getStatus()){
		//return;
		printWebPage(ms);
	}
	// Servlet-------------------------------------------------
	else{
		System.out.print("<p> The metadataStats fails</p>");
	}
	//System.out.println("\nTotal Number of Documents: "+ms.getDocNum());
	//System.out.println("\n system ends");
	//System.out.println(ms.metadataNameList.size());

    }

    public void printWebPage(MetadataStats ms){
    	//long startTime=System.currentTimeMillis();	
    	DataMaker dm = new DataMaker(ms);
    	String[] elements =  new String [ms.metadataNameList.size()];
        
    	for(int i = 0; i<elements.length; i++){
    		elements[i] = (String)ms.metadataNameList.get(i);
        }
	
        Arrays.sort(elements);
    	String[] metaDataElementName = (String[])elements.clone();	
    	String[] ids = dm.getDocumentIDs("archivedir");
        
    	ArrayList alist = new ArrayList();
    	for(int a = 0; a<elements.length; a++){
    		alist.add(dm.getMetadataRows(elements[a]));
    	}
    	//System.out.println(alist.size()+ " " +ids.length+" "+metaDataElementName.length);   	
    	PrintHTML phtml = new PrintHTML(ms);

    	//?????????????????????????????????????????????????????????????????????????/
    	HashMap hp = ms.getMetadataSetMap();
    	Collection c = hp.values();
    	Iterator i = c.iterator();
    	
    	while(i.hasNext()){
    		MetadataSet mds  = (MetadataSet)i.next();
    		//if(!mds.getName().equals("extracted")){
    		ArrayList elementList = mds.getIndexsList();
    		String[] mdsElement = new String[elementList.size()];
    		for(int x = 0; x<mdsElement.length; x++){
    			mdsElement[x]=(String)elementList.get(x);
    		}
    		Arrays.sort(mdsElement);
    		
        	String[] metaDataElement = (String[])mdsElement.clone();	
        	String[] idList = dm.getDocumentIDs("archivedir");
            
        	ArrayList datalist = new ArrayList();
        	for(int a = 0; a<mdsElement.length; a++){
        		datalist.add(dm.getMetadataRows(mdsElement[a]));
        	}
        	phtml.generateAllPossibleGraph(datalist,idList,metaDataElement,mds.getName());
    		//}
    	}
    	
    	//??????????????????????????????????????????????????????????????????????????
    	//long startTime=System.currentTimeMillis();
    	//phtml.generateAllPossibleGraph(alist,ids,metaDataElementName,"both");
    	//////////////////////////////////
    	//Now generate chart  for individaul mds
    	//????????????????????????????????????????????????????????//
    	
    	//long endTime=System.currentTimeMillis();
    	//System.out.println("Total time taken for generating overall Statistics Page "+(endTime-startTime)+ " millisec");

    	phtml.generateOverallStatisticsPage(hp);

    	Collection cx = ms.getMetadataSetMap().values();
    	Iterator ix = cx.iterator();

    	while(ix.hasNext()){
    		MetadataSet temp = (MetadataSet)ix.next();
    		phtml.WriteMetadataSetDetailHTML(temp);
    		phtml.generateMetadataElementDetailPage(temp);
    	}
    }
 
    
    public MetadataStats getElementClass()
    {
	return ms;
    }
    
    /*
    public static void main(String[] args) {
    	DescribeMessenger dm = new DescribeMessenger();
    	System.out.println("test");
    	dm.describe("","","","");
    	dm.describeMatadata("sampleco","http://researchcommons.waikato.ac.nz/cgi/oai2.cgi");
    }
    */
}

